#include "allvars.ash"
#include "ddef.asm"

; Boot sector format:
;  0:  jp boot         < need for boot
;  3:  55,AAh          < need for config
;  5...  config
; 7F:  5Ah             < need for boot&config
; cyclic checksum = 1  < need for boot&config

; Config record format:
; db type
; db size
; ...
; datum
; ....

; Initial channel assigment:
;   Ch0  - floppy
;   Ch1  - Edisk
;   Ch2  - HDD

try_lod:ld      (ix+_RQCOM),_RESET
        call    dmon_e
        ret     nz
        ld      (ix+_RQCOM),_RECAL
        call    dmon_e
        ret     nz
        ld      (ix+_RQCOM),_READ
        ld      hl,FLP_load
        ld      de,RQCOM+1
        ld      bc,7
        ldir
        jp      dmon_e
FLP_load:
        db      1               ; 1 block
        dw      0               ; 0 track
        dw      1               ; 1st block (1-based)
        dw      8000h

cfg_load::ld    (ix+_RQCHAN),0        ; try to load from floppy (ch0)
        call    try_lod
        jr      z,cl_chk
        ld      (ix+_RQCHAN),2        ; try to load from hard (ch2)
        call    try_lod
        ret     nz                    ; can't load config
cl_chk: ld      hl,8000h
        ld      b,80h
        xor     a
clc_lp: add     a,(hl)
        adc     a,0
        inc     hl
        djnz    clc_lp
        dec     a
        ret     nz
        ld      a,(8003h)
        cp      55h
        ret     nz
        ld      a,(8004h)
        cp      0AAh
        ret     nz
;        ld      a,(807Fh)
;        cp      5Ah
;        ret     nz

conf::  ld      a,0C9h          ; RET
        ld      (SYS_ON##),a
        ld      (SYS_OFF##),a
        ld      iy,8005h
cfg_i:  ld      a,(iy)
        vcall   <_done,term,skip,load,ass,cinit,exec,go,format>
        or      a
        jr      z,done
err:    call    prn##
        db      0dh,0ah,'Config error ',0
        call    phex##
        call    prn
        db      'startup aborted!',0dh,0ah,0
        scf
        jr      st_ret

_done:  pop     af
done:   ld      b,0
done1:  ld      c,(iy+1)
        add     iy,bc
        jr      cfg_i
term:   pop     af
        or      a
st_ret: ld      a,0F3h          ; DI
        ld      (SYS_ON##),a
        ld      (SYS_OFF##),a
        ret

skip:   pop	af
	ld      b,(iy+2)        ; db hi(size)
        jr      done1

g_dat:  ex      (sp),hl
        ld      e,(hl)
        inc     hl
        ex      (sp),hl
        ld      d,0
        push    iy
        ex      (sp),hl
        add     hl,de
        ex      (sp),hl
        pop     de
        ret

load:   call    g_dat           ; db num of 128-bytes blocks
        db      3               ; ds ... chanal for load
        ld      a,_SETCH
        ld      c,9
        CALL    PSCHK##
        or      a
        ret     nz
        ld      de,RQCHAN
        ld      hl,RQimg
        ld      bc,9
        ldir
        ld      a,(iy+2)
        ld      (RQBLN),a
dmon_e: push    iy
        call    DISK##
        pop     iy
        or      a
        ret
RQimg:  db      9                    ;RQCHAN  - 
        db      _READ                ;RQCOM   - 
        db      1                    ;RQBLN   -  
        dw      0                    ;RQTRACK - 
        dw      2                    ;RQBLOCK - 
        dw      8080h                ;RQBADR  -  

ass:    call    g_dat           ; db drive (or -1 for chanal only)
        db      4               ; db chan_num (+hi bit - not set ch)
        ld      c,(iy+3)        ; ds ... < chanal itself
        bit     7,c
        jr      nz,not_ass_ch
        push    de
        ld      a,_SETCH
        call    PSCHK
        pop     de
        or      a
        ret     nz
        jr      ass_it
not_ass_ch:
        ld      de,0
ass_it: ld      b,(iy+2)
        bit     7,b
        ret     nz
        ld      c,(iy+3)
        set     7,b
        res     7,c
        jp      bios+33h


cinit:  call    g_dat           ; db 'str',0
        db      2
cini_l: ld      a,(de)
        inc     de
        or      a
        ret     z
        rst     20h
        jr      cini_l

exec:   ld      l,(iy+2)        ; dw proc, call it (if 0 - call iy+4)
        ld      h,(iy+3)
	ld	a,h
	or	l
	jr	nz,exe_ok
	call	g_dat
	db	4
	ex	de,hl
exe_ok: jp      (hl)

go:     call    g_dat
        db      2
        ld      a,(iy+1)        ; db 'run str'
        sub     2
        ld      (CCP+7),a
        ld      c,a
        ld      b,0
        ex      de,hl
        ld      de,CCP+8
        ldir
        xor     a
        ret

format: ld      a,(iy+2)        ; db chan num
        ld      (RQCHAN),a      ; db track num
        ld      a,(iy+3)        ; db filler
        ld      (RQTRACK),a
        ld      a,(iy+4)
        ld      (RQBLN),a
        ld      (ix+_RQTRACK+1),0
        ld      (ix+_RQBADR),2
        ld      (ix+_RQCOM),_FORMAT
        jp      dmon_e

end
