#include "allvars.ash"
#include "ddef.asm"

HD_DRV::
hd_dat   equ     0fh
hd_err   equ     2fh
hd_seccnt equ    4fh
hd_sect  equ     6fh
hd_cyllo equ     8fh
hd_cylhi equ     0afh
hd_head  equ     0cfh
hd_comm  equ     0efh

DW      HD_RES                  ; reset HD
DW      HD_SEEK                 ; seek HD
DW      HD_FRM                  ; format HD
DW      HD_REC                  ; recalibrate HD
DW      HD_RD                   ; read
DW      HD_NOP
DW      HD_WR                   ; write

HD_tst: bit     6,(IY+DDTYP)    ; 6 bit - checked
        ret     nz
        bit     7,(IY+DDTYP)    ; 7 bit - no disk
        jr      nz,_1err
        ld      bc,hd_comm
        in      a,(c)
        inc     a
        jr      z,_1err
        ld      d,0
        call    HD_wt1
        or      a
        jr      nz,_1err
        ld      bc,hd_cylhi
        ld      a,55h
        out     (c),a
        cpl
        ld      c,hd_cyllo
        out     (c),a
        ld      c,hd_cylhi
        in      a,(c)
        cp      55h
        jr      nz,_1err
        ld      c,hd_cyllo
        in      a,(c)
        cp      0AAh
        jr      nz,_1err
        set     6,(IY+DDTYP)
hd_scl: ld      hl,(RQCYL)
        ld      bc,hd_cyllo
        out     (c),l
        ld      c,hd_cylhi
        out     (c),h
        ret

_1err:  set     7,(IY+DDTYP)
        pop     af
nr_err: ld      a,_NRDY
        or      a
        ret

HD_RES:
HD_REC: call    HD_tst
        ld      bc,hd_comm
        ld      a,10h
        out     (c),a
HD_wait::ld     d,64
hd_wt1: ld      bc,hd_comm
        ld      hl,0
hd_w_lp:in      e,(c)
        bit     7,e
        jr      z,hd_w_dn
        dec     hl
        ld      a,l
        or      h
        jr      nz,hd_w_lp
        dec     d
        jr      nz,hd_w_lp
        jr      nr_err

hd_w_dn:bit     0,e
        jr      nz,is_hd_err
;        and     50h
;        sub     50h
;        jr      nz,hd_w_lp
HD_NOP: xor     a
        ret
is_hd_err:
        ld      bc,hd_err
        in      b,(c)
        bit     0,b
        ld      a,_NO$ADDR$MARK
        ret     nz
        bit     2,b
        jr      nz,hd_err_ab
        bit     4,b
        ld      a,_NO$DATA
        ret     nz
        bit     6,b
        jr      z,hd_sh1
        bit     2,e
        ld      a,_CRC$ERR
        ret     z
        xor     a
        ret
hd_sh1: bit     7,b
        ld      a,_FATAL$ERROR
        ret     nz
        ld      a,_HRDERR
        ret
hd_err_ab:
        bit     6,e
        ld      a,_NRDY
        ret     nz
        bit     5,e
        ld      a,_WR$PROT
        ret     nz
        bit     4,e
        ld      a,_IOERR
        ret     nz
        ld      a,_COMERR
        ret

HD_SEEK:call    HD_tst
        ld      hl,(RQCYL)
        ld      a,h
        or      l
        jr      z,HD_REC
        call    hd_scl
        ld      c,hd_comm
        ld      a,70h
        out     (c),a
        jr      HD_wait


hd_set_rw:
        ld      a,(RQHEAD)
        or      0a0h
        ld      bc,hd_head
        out     (c),a
        ld      a,(RQSECT)
        inc     a
        ld      c,hd_sect
        out     (c),a
        ld      a,d
        ld      c,hd_seccnt
        out     (c),a
        call    hd_scl
        ld      c,hd_comm
        ret

HD_RD:  call    HD_tst
        ld      d,1
        call    hd_set_rw
        ld      a,20h
        out     (c),a
        call    HD_wait
        or      a
        ret     nz
        ld      hl,(DCBUF)
        ld      bc,hd_dat
        inir
        inir
        ld      bc,hd_comm
        in      a,(c)
        and     8
        ret     z
        ld      a,_OVERRUN
        ret

hd_setwr:
        call    hd_set_rw
        ld      a,30h
        out     (c),a

hd_w_w: in      e,(c)
        bit     0,e
        jp      nz,is_hd_err
        bit     3,e
        jr      z,hd_w_w
        ld      bc,hd_dat+256
        ret

HD_WR:  call    HD_tst
        ld      d,1
        call    hd_setwr
        ld      hl,(DCBUF)
        dec     d
        inc     hl

hd_wlp:
        ld      a,(hl)
        out     (c),a
        dec     b
        dec     hl
        ld      a,(hl)
        out     (c),a
        inc     b
        inc     hl
        inc     hl
        inc     hl

        push bc
        ld bc,hd_comm
        in a,(c)
        pop bc

        dec     d
        jr      nz,hd_wlp

hd_wwait:
        ld      bc,tlrd
hd_wlpd:in      a,(c)
        and     40h
        jr      z,hd_wlpd
        jp      HD_wait


HD_FRM:
        call    HD_SEEK
        or      a
        ret     nz
        ld      d,(IY+DSECTT)
        call    hd_setwr

hd_f2:  ld      a,(RQBLN)
        out     (c),a
        dec     b
        ld e,b
hd_f1:
        push bc
        ld bc,hd_comm
        in b,(c)
        pop bc

        out     (c),a
        ;ld      e,b


        dec     e
        jr      nz,hd_f1

        ld      bc,tlrd
hd_f3:  in      a,(c)
        and     40h
        jr      z,hd_f3

        ld      bc,hd_comm
        in      e,(c)
        bit     0,e
        jp      nz,is_hd_err
        bit     3,e
        jp      z,HD_wait
        ld      bc,hd_dat+256
        jr      hd_f2
end
