.z80

magic   equ     2A56h
tlrd    equ     7ffdh
adrd    equ     7dfdh

ncvt:   ld      b,1
nc_l:   cp      b
        jr      c,nc_d
        inc     a
        sla     b
        jr      nc,nc_l
nc_d:   cpl
        ld      b,a
        ret

_get:   ;call    ncvt
        ld      c,0feh
        in      a,(c)
        res     7,b
        in      c,(c)
        sla     c
        sla     c
        sla     c
        sla     c
        and     0fh
        or      c
        ret

get:    ex      (sp),hl
        push    bc
        ld      b,(hl)
        inc     hl
        call    _get
        pop     bc
        ex      (sp),hl
        ret

_put:   ;call    ncvt
        ld      c,0e7h
        out     (c),a
        rra
        rra
        rra
        rra
        res     7,b
        out     (c),a
        ret

put:    ex      (sp),hl
        push    af
        push    bc
        ld      b,(hl)
        inc     hl
        call    _put
        pop     bc
        pop     af
        ex      (sp),hl
        ret

$c      macro   sh
$$l     defl    sh
$$m     defl    1
        rept    8
        if      $$l lt $$m
        db      not $$l
        exitm
        endif
$$l     defl    $$l + 1
$$m     defl    $$m * 2
        endm
        endm

P       macro   sh
        call    put
        $c      sh
        endm

G       macro   sh
        call    get
        $c      sh
        endm

set:    ex      (sp),hl
        G       flags
        or      (hl)
        inc     hl
        P       flags
        ex      (sp),hl
        ret

res:    ex      (sp),hl
        G       flags
        and     (hl)
        inc     hl
        P       flags
        ex      (sp),hl
        ret

S       macro   bit
        call    set
        db      bit
        endm

R       macro   bit
        call    res
        db      not bit
        endm

B       macro   bit
        G       flags
        and     bit
        endm

;----------------------------
; main program
;
int:	push    hl              ; save first for retn emulation
        push    af
        push    bc

        ld      bc,tlrd
        in      a,(c)
	add	a,a
        jr      c,is_key
        pop     af
        pop     bc
        pop     hl
        jp      magic

is_key: 
        push    de
        ld      a,0ffh
        in      a,(0feh)
        and     1fh
        cp      15h             ; will be init?
        call    nz,kb_init      ; initialise if not
        B       e0_flag         ; prev scan is e0?
        jp      nz,is_e0
        call    kb_read
        cp      0e0h
        jr      nz,no_e0
        S       e0_flag
        jr      done
no_e0:  cp      0e1h
        jp      z,pause
        add     a,a
        call    $sub1
        cp      3bh
        jr      nc,is_ext
dk_2:   ld      hl,sc_tab-1
        add     a,l
        ld      l,a
        jr      nc,$+3
        inc     h
dk_1:   ld      a,(hl)
do_key: call    onoff
done:   ld      hl,45edh        ; create retn instraction on stack
        push    hl
        ld      hl,11
        add     hl,sp
        ld      a,(hl)
        and     0C0h
        jr      nz,kk_1
        ld      d,(hl)
        dec     hl
        ld      e,(hl)
        dec     de
        dec     de
        ld      (hl),e
        inc     hl
        ld      (hl),d
kk_1:   ld      hl,0
        add     hl,sp
        pop     af

        pop     de
        pop     bc
        pop     af

        ex      (sp),hl
        ret

is_ext: cp      45h
        jr      z,is_num
        B       num_on
        ld      a,l
        jr      z,do_ext
        ld      hl,tab1
        ld      bc,Ltab1
        cpir
        jr      z,do_num
do_ext: sub     3bh-base_ram
        call    ncvt
        call    _get
        jr      do_key
do_num: dec     c
        ld      a,SS+LitM
        jp      m,do_key
        ld      hl,sc_tab+1
        add     hl,bc
        jr      dk_1
is_num: bit     7,e
        jr      nz,done
        G       flags
        xor     num_on
        P       flags
        jr      done

is_e0:  R       e0_flag
        call    kb_read
        add     a,a
        inc     a
        call    $sub1
        cp      38h
        jr      z,dk_2
        cp      1dh
        jr      z,dk_2
        ld      hl,tab2
        ld      bc,Ltab2
        cpir
        jr      nz,done
        ld      a,59h
        add     a,c
        jr      do_ext

pause:  call    kb_read
        call    kb_readw
        call    kb_readw
        call    kb_readw
        call    kb_readw
        call    kb_readw
	dec	a
	call	z,kb_init
	jp	done

$sub1:  ld      l,a
        sbc     a,a
        set     0,a
        ld      e,a
        ld      a,0
        jr      c,$2    ; break code -> clear last key
        G       last_k  ; make code
        cp      l
        jr      z,done1 ; auto rep
        ld      a,l     ; else save it
$2:     P       last_k
        srl     l
        ld      a,l
        ret
done1:  pop     af
        jp      done

tab1:   db 52h,49h,48h,47h,4dh,4ch,4bh,51h,50h,4fh,53h ; 0987654321.
Ltab1   equ     $-tab1

tab2:   db 35h,1ch,4dh,51h,49h, 50h,48h,4fh,47h, 4bh,53h,52h,37h
         ; /   Ent -> PgDn PgUp |   A   End Home <-  Del Ins PrtScr
         ;                      V   |
Ltab2   equ     $-tab2
;------------------------------
; emulation of key
;
onoff:  push    af              ; E is 1-on, -1-off.
        add     a,a             ; A 0-2 is scan num
        push    af              ; A 3-5 is ret num
        call    m,cs_oo         ; A 6 is CS, 7 is SS
        pop     af
        call    c,ss_oo
        pop     af
        and     3fh
        cp      3fh
        ret     z
oo_key: ld      d,a
        call    $s1
        push    af
        ld      a,d
        rra
        rra
        rra
        call    $s1
        pop     bc
        ld      c,0feh
        in      d,(c)
        bit     7,e
        jr      nz,$off
        and     d
$1:     ld      c,0e7h
        out     (c),a
        ld      a,0ffh
        ld      bc,0fefeh
$lp:    in      d,(c)
        and     d
        rlc     b
        jr      c,$lp
        ld      bc,0e7h
        out     (c),a
        ret
$off:   cpl
        or      d
        jr      $1
$s1:    and     7
        ld      b,a
        inc     b
        ld      a,0ffh
$s1l:   rla
        djnz    $s1l
        ret
cs_oo:  G       cs_state
        add     a,e
        push    af
        P       cs_state
        ld      d,cshift
$as_s:  pop     af
        add     a,a
        sub     e
        dec     a
        ld      a,d
        jp      z,oo_key
        ret
ss_oo:  G       ss_state
        add     a,e
        push    af
        P       ss_state
        ld      d,sshift
        jr      $as_s

;-------------------------------
; keyboard system init
;
kb_init:ld      b,2bh
$ki_l1: push    bc
        ld      a,base_ram-1
        add     a,b
        call    ncvt
        ld      a,spare
        call    _put
        pop     bc
        djnz    $ki_l1
        ld      hl,init_s
$ki_l2: ld      a,(hl)
        inc     hl
        add     a,base_ram
        jr      c,$ki1
        call    ncvt
        ld      a,(hl)
        inc     hl
        call    _put
        jr      $ki_l2
$ki1:   xor     a
        P       flags
kb_res: ld      bc,0ffe7h
        ld      a,15h
        out     (c),a
        ld      a,b
        dec     b
$ki_l:  out     (c),a
        rlc     b
        jr      c,$ki_l
        xor     a
        P       cs_state
        P       ss_state
        P       last_k

        G       flags
        and     num_on
        P       flags

        ret

;--------------------------------
; read keyboard
;
kb_readw:push   bc
        ld      bc,tlrd
kbwt:   in      a,(c)
        rla
        jr      nc,kbwt
        pop     bc
kb_read:push    bc
        ld bc,adrd
        in a,(c)
        in a,(c)
        ld b,8
kbr_2:  rra
        rl c
        djnz kbr_2
        ld a,c
        ld b,16
        djnz $
        ld bc,adrd
        in c,(c)
        pop     bc
        ret

;---------------------------------
;  work cells
;
        aseg
        org     1

ss_state: ds 1
cs_state: ds 1
flags:    ds 1
last_k:   ds 1

base_ram:
        cseg

;   bit sets for flags
e0_flag  equ    1
num_on   equ    2

;---------------------------------
; same equ
;
cshift   equ    0
sshift   equ    0fh

;----------------------------------
; const for scans table
;
CS	equ	40h
SS	equ	80h

_def	macro	name,adr,ret
Lit&name equ	adr+ret*8
	endm
def	macro	p
	irp	n,<p>
	_def	n
	endm
	endm

def     <        <A,1,0>,<Q,2,0>,<1,3,0>,<0,4,0>,<P,5,0>,<Ent,6,0>,<Sp,7,0>>
def     <<Z,0,1>,<S,1,1>,<W,2,1>,<2,3,1>,<9,4,1>,<O,5,1>,<L,6,1>>
def     <<X,0,2>,<D,1,2>,<E,2,2>,<3,3,2>,<8,4,2>,<I,5,2>,<K,6,2>,<M,7,2>>
def     <<C,0,3>,<F,1,3>,<R,2,3>,<4,3,3>,<7,4,3>,<U,5,3>,<J,6,3>,<N,7,3>>
def     <<V,0,4>,<G,1,4>,<T,2,4>,<5,3,4>,<6,4,4>,<Y,5,4>,<H,6,4>,<B,7,4>>

;----------------------------------
; scans table
;
spare      equ     3fh
sc_tab: db CS+Lit1, Lit1, Lit2, Lit3, Lit4, Lit5, Lit6, Lit7, Lit8, Lit9, Lit0
        db SS+LitJ, SS+LitK, CS+Lit0, CS+Lit3, LitQ, LitW, LitE, LitR, LitT
        db LitY, LitU, LitI, LitO, LitP, SS+Lit8, SS+Lit9, LitEnt, SS+spare, LitA
        db LitS, LitD, LitF, LitG, LitH, LitJ, LitK, LitL, SS+LitO, SS+LitP
        db SS+LitX, CS+spare, SS+Lit0, LitZ, LitX, LitC, LitV, LitB, LitN, LitM
        db SS+LitN, SS+LitM, SS+LitC, CS+spare, SS+LitB, SS+CS+spare, LitSp, CS+Lit2

init_s: db 0dh,CS+Lit7, 10h,CS+Lit5, 12h,CS+Lit8, 15h,CS+Lit6, 11h,Lit0
        db 24h,CS+Lit7, 21h,CS+Lit5, 28h,CS+Lit8, 25h,CS+Lit6
        db 29h,LitEnt,  2ah,SS+LitV, 0fh,SS+LitJ, 13h,SS+LitK
        db 0ffh

end int
