#include "allvars.ash"

;------------------------------------------
; RESET - reset drv. A 0-screen, 1-mode, 2-full reset
;
RESET:: sub     2
        jr      nz,_ret_ok_
        ld      (T_FLAG),a
        ld      (ix+_CUR_VAL),'_'
        ld      hl,41C0h
        ld      (SCR_ADR),hl
        ret

;------------------------------------------
; OUT_CH -  output char.  A - char
;
_out_ch_::ex      af,af'
        xor     a
        ex      af,af'
        ld      c,a
        ld      hl,(SCR_ADR)
        bit     _over,(ix+_flag1)
        jr      z,_put_ch_
        cp      ' '
        jr      z,skp_ch
        ld      a,' '
        cp      (hl)
        jr      z,_put_ch_
        ex      af,af'
        ld      a,CsT_over
        ex      af,af'
        db      3eh             ; ld a,n (skip next command)
_put_ch_: ld      (hl),c
skp_ch: bit     _trans,(ix+_flag1)
        jr      nz,done_ch
        call    attr?
        ld      a,(COLOR_M)
        ld      (hl),a
done_ch:ex      af,af'
        ret
;
attr?:  ld      a,h             ; convert screen address to attribute
        or      0C0h
        jr      adv_1

;------------------------------------------
; ADV_POS - advance position
;
ADV_POS::ld      hl,(SCR_ADR)
        call    adv
        ld      (SCR_ADR),hl
_ret_ok_: xor     a
        ret

adv:    ld      a,h
adv_1:  xor     20h
        ld      h,a
        and     20h
        ret     nz
        inc     hl
        ret

;------------------------------------------
; GOTO_XY - direct addressation. C - x, B - y
;
GOTO_XY::call    _goxy_
        ld      (SCR_ADR),hl
        xor     a
        ret

_goxy_:  xor     a
        ld      h,a
        rrc     c       ; c>>1 & set CF
        rra
        rra
        rra             ; a is 20h if CF
        or      41h
        ld      l,b     ; hl is y
        ld      b,a     ; bc is (x>>1)+(x&1?2000h:0)+41C0h
        set     7,c
        set     6,c
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl   ; y*=64
        add     hl,bc   ; hl is address
        ret

;------------------------------------------
; CURS_OO - cursor turn on/off. A - curs state
;
CURS_OO::or      a
        ex      af,af'
        ld      hl,(SCR_ADR)
        bit     _TF_curatr,(ix+_T_flag) ; if flex attrib...
        call    nz,attr?                ;  deal with attrib address
        ex      af,af'
        ld      a,(SAV_CUR)             ; cursor off - restore buf contents
        call    nz,curs_on
        ld      (hl),a
        xor     a
        ret
curs_on:ld      a,(hl)
        ld      (SAV_CUR),a
        ld      a,(CUR_VAL)
        bit     _TF_curatr,(ix+_T_flag) ; if no flex attrib...
        ret     z                       ;  jump - simple put
        xor     (hl)                    ;  else - xor'ed value
        ret

;------------------------------------------
; CONV_COL - color convert. B-ink, C-paper, A-color
;
CONV_COL::ld     (COLOR_M),a
        xor     a
        ret

;------------------------------------------
; SET_CURS - set cursor scans. B-beg line, C-end line
;
SET_CURS::set    _TF_curatr,(ix+_T_flag) ; presume 'flex attr' mode
        ld      a,7
        cp      c
        jr      nz,sc_1
        cp      b
        ld      a,'_'
        jr      z,set_chr
        inc     b
        dec     b
        ld      a,3fh
        jr      z,set_atr
sc_1:   bit     2,b
        ld      a,','+80h       ; abs code of ''
        jr      nz,set_chr
        ld      a,'/'+80h       ; abs code of ''
        bit     2,c
        jr      z,set_chr
        dec     a
set_chr:res     _TF_curatr,(ix+_T_flag)
set_atr:ld      (CUR_VAL),a
        xor     a
        ret

ff_err: ld      a,Cs_unkno
        ret

;------------------------------------------
; CIO_MUX - mux input.  B - func
;
CIO_MUX::ex      af,af'
        ld      a,b
        vjmp    <ff_err,M_ver,M_cursC,M_cursA>

M_ver:  ld      hl,100h         ; text mode driver version
        xor     a
        ret

M_cursC equ     set_chr         ; set char from A for cursor image
M_cursA:set     _TF_curatr,(ix+_T_flag) ; set attrib for cursor image
        jr      set_atr


;---- aux proc for clr/roll procs ---------
_size:  ld      c,e     ; size of set of chars.
        srl     c       ; in E-dx, out C-size1, B-size2
        ld      b,c
        ret     nc
        inc     c
        ret

_prep:  ex     af,af'   ; check dx&dy, determ. screen addr, evaluate sizes
        ld      a,Cs_erpar
        scf
        inc     d
        dec     d
        ret     z
        inc     e
        dec     e
        ret     z       ; if dx or dy is 0 - error
        call    _goxy_
        call    _size
        ex      af,af'
        or      a
        ret

;----------- roll/clr loop ---------------
;
clr_roll:ld     a,0c3h          ; HL-buf, BC-sizes, DE- -shift
        ld      (cl_sw),a      ;  C'-char(if need), B'-line count
        exx
cr_lp:  ld      a,c             ; is value
        exx
        or      a
        call    nz,_clrmov
        bit     _trans,(ix+_flag1)
        jr      nz,cr_sc
        push    hl
        ex      af,af'
        call    attr?
        ld      a,(COLOR_M)
        call    _clrmov
        ex      af,af'
        pop     hl
cr_sc:  xor     a
        sbc     hl,de
        exx
        djnz    cr_lp
        exx
        ret

_clrmov:push    de
        push    hl
        call    cl_sw           ; call first part
        pop     hl
        push    hl
        ex      af,af'
        call    adv
        ex      af,af'
        call    cl_sw           ; call second part
        pop     hl
        pop     de
        ret

;------------------------------------------
; CLW_WIND - clear window. BC-y1&x1, DE-dy&dx, A-char
;
CLW_WIND::call   _prep
        ret     c
        ld      e,a
        push    de
        exx
        pop     bc      ; B'-dy, C'-char
clr_:   exx
        ld      de,_clr_ ; form 'jp _clr' command into ed_buf
        ld      (cl_sw +1),de
        ld      de,-64
        call    clr_roll
        ret

_clr_:  push    bc
        dec     c
        jp      m,_cl_dn
        ld      (hl),a
        jr      z,_cl_dn
        ld      d,h
        ld      e,l
        inc     de
        ld      b,0
        ldir
_cl_dn: pop     bc
        ld      e,b
        ld      b,c
        ld      c,e
        ret

;------------------------------------------
; ROL_WIND - roll window. BC-y1&x1, DE-dy&dx, A-dir:
;                          0-UP,1-DOWN,2-RIGHT,3-LEFT
;
ROL_WIND::call    _prep
        ret     c       ; HL - addr, BC-size 2&1
        cp      2
        jr      nc,roll_hor ; horizontal roll
        rra
        dec     d
        jr      z,rol_sc; 1 line don't roll
        ld      a,d
        exx
        ld      b,a     ; B' is line count
        ld      c,1     ; C' not zeroed
        exx
        ld      de,_move; form 'jp _move' command into ed_buf
        ld      (cl_sw +1),de
        ld      de,-64  ; presume 'roll up'
        jr      nc,is_upp ; jump if was 'roll up' command
        ex      de,hl
        ld      l,a
        ld      h,0
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl
        add     hl,hl   ; .. and multiply it by 64
        add     hl,de
        ld      de,64   ; HL -> end of window, DE - 64
is_upp: push    bc
        call    clr_roll
        pop     bc
        exx
rol_sc: ld      bc,256+' '
        jr      clr_

roll_hor:ld     a,b
        ld      bc,adv  ; INC proc in 'roll right' case
        jr      z,rh_ok
        call    add_hl_a
        ld      bc,deg  ; else DEC proc
        bit     0,e
        call    z,deg
rh_ok:  ld      (cl_sw +1),bc
        ld      a,0c3h
        ld      (cl_sw ),a
rh_lp:  ld      a,' '
        push    hl
        call    rh_do   ; pull symbol ' ' to screen line
        pop     hl
        bit     _trans,(ix+_flag1)
        jr      nz,rh_sc
        push    hl
        call    attr?
        ld      a,(COLOR_M)
        call    rh_do   ; and pull attribute (if need)
        pop     hl
rh_sc:  ld      bc,64
        add     hl,bc
        dec     d
        jr      nz,rh_lp
        xor     a
        ret

rh_do:  ld      b,e     ; is dx
        ld      c,a     ; is pulled value
rh_pull:ld      a,(hl)  ; pull loop
        ld      (hl),c
        ld      c,a
        call    cl_sw
        djnz    rh_pull
        ret

deg:    ld      a,h
        xor     20h
        ld      h,a
        and     20h
        ret     z
        dec     hl
        ret

_move:  push    de      ; C-size, DE-disp. HL,DE,BC,A-preserve
        push    bc
        push    hl
        push    hl
        or      a
        sbc     hl,de
        pop     de
        ld      b,0
        inc     c
        dec     c
        jr      z,_mv1
        ldir
_mv1:   pop     hl
        pop     bc
        ld      e,b
        ld      b,c
        ld      c,e
        pop     de
        ret

.comment @
;------------------------------------------
; IMG_PG - put/get image. BC-y1&x1, DE-dy&dx, HL-buf
;                         A.[0:6]-page, A.[7]-func:
;                               0-get/1-put image
;
IMG_PG:: push    hl
        call    _prep
        ex      af,af'
        and     1fh
        call    ed_set
        ex      af,af'
        ld      b,d     ; B-dy
        ld      c,e     ; C-dx
        pop     de      ; DE-buf addr, HL-screen addr
        ret     c       ; ret if error into '_prep'
        add     a,a
        exx
        ld      hl,put_prg
        jr      c,put_im
        ld      hl,get_prg
put_im: ld      de,ed_buf
        ld      bc,len_prg
        ldir
        exx
        ex      de,hl   ; HL-buf, DE-screen
img_lp: push    bc
        push    de
i_llp:  call    ed_buf  ; move one char
        push    de
        ex      de,hl
        call    attr?
        ex      de,hl
        call    ed_buf  ; ... and one attribute
        pop     de
        ex      de,hl
        call    adv
        ex      de,hl
        dec     c
        jr      nz,i_llp
        exx             ; next line
        pop     hl
        ld      bc,64
        add     hl,bc
        push    hl
        exx
        pop     hl
        pop     bc
        djnz    img_lp
        xor     a
        ret

get_prg:ld      a,(de)
        sta     pp
        ret

put_prg:lda     pp
        ld      (de),a
        ret

len_prg equ     put_prg-get_prg
@

end
