.z80

SPtop   equ     8000h

        org 0
        jp run
        ld hl,sinc_pal          ; call 3
        ret

        org     8
        jp  set_pg              ; rst  8
        jp  _set_pg             ; call 0bh

        org     10h
        jp      goboot          ; rst  10h
        jp      derr            ; call 13h

        org     18h
        jp      set_pal1        ; rst  18h
set_pal: exx                    ; call 1bh
        ld      c,0feh
        jr      _sp_

        org     20h
        jp      int13##         ; rst  20h
        jp      pexit           ; jp   23h

        org     28h
        jp      disk##          ; rst  28h
        jp      pexit0          ; jp   2bh

        org     30h
        jp      r_key##         ; rst  30h
        jp      tst_cks         ; call 33h

        org 38h
        scf
	ei
        ret

page    macro   n
;        call    set_pg
	rst	8
        db      n
        endm

;-------------------------
; set pallet from HL
;
set_pal1: exx
       ld   c,0beh
_sp_:  exx
       ld   a,0f0h
       ld   bc,7dfdh
       ei
       halt
       di
spl:   exx
       cp   0f8h
       jr   nz,sp_ok
       res  3,c
sp_ok: out  (c),a
       exx
       outi
       inc  b
       inc  a
       jr   nz,spl
       ret

;-------------------------
; set page
;
set_pg: ex      (sp),hl
        ld      a,(hl)
        inc     hl
        ex      (sp),hl
_set_pg:push    bc
        push    af
        rra
        rra
        rra
        and     3
        ld      bc,0fdfdh
        out     (c),a
        pop     af
        and     7
        or      10h
        ld      bc,7ffdh
        out     (c),a
        pop     bc
        ret

;------------------------------
; ret NZ if turbo
;
tst_tur: ld hl,0
        ei
        halt
        or      a
tt_1:   inc     hl
        jr      nc,tt_1
        di
        ld      a,10h
        cp      h                   ; c-turbo
        ret

;------------------------------
; evaluate check sum of upper page in A
;
tst_cks:ld      hl,0c000h
        ld      de,1
        xor     a
tcs_1:  add     a,(hl)
        adc     a,0
        add     hl,de
        jr      nc,tcs_1
        ret

;--------------------------------
; load header
;
l_head: push    de
        push    hl
        push    bc
lh_back:ld      ix,h_buf
        ld      de,17
        xor     a
        call    LD_bytes##
        inc     a
        jr      z,lh_back
        jr      nc,l_ret
        ld      a,(h_buf)
        cp      3
        jr      nz,lh_back
        ld      a,(h_buf+1)
        sub     '0'
        cp      7
        jr      nc,lh_back
l_ret:  pop     bc
        pop     hl
        pop     de
te_:    ccf
        ret     nc
        pop     hl
        ld      hl,Mtlerr##
err:    xor     a
        out     (0feh),a
        ld      (mes_adr),hl
        set     _nmes,(iy+_flag)
        rst     10h

;--------------------------------
; load body
;
l_body: ld      ix,0c000h
        ld      de,4000h
l_body1:ld      a,0ffh
        call    LD_bytes##
        jr      te_

;-------------------------

INCLUDE VARS.INC


pal:
DB 0FEH,5DH,65H,42H,78H,53H,7EH,37H,0DH,40H,6AH,0A1H,7FH,3FH,21H,58H
sinc_pal:
DB 0FFH,0FEH,0FDH,0FCH,0FBH,0FAH,0F9H,0F8H,0FFH,0F6H,0EDH,0E4H,0DBH
DB 0D2H,0C9H,0C0H

run:    ld sp,SPtop
        di
        xor     a
        out     (0feh),a
        im 1
        ld     iy,iy_base
        ld     (iy+_flag),0
;        ld     (iy+_mash),15
;        ld     (iy+_pcnt),1
;        ld     (iy+_switch),0c3h
     ; test for RU7
        ld      hl,0c000h
        page    9
        ld      e,(hl)
        ld      (hl),0aah
        page    1
        ld      d,(hl)
        ld      (hl),55h
        page    9
        ld      a,0aah
        xor     (hl)
        jr      z,is_ru7
        set     _ry7,(iy+_flag)
is_ru7: ld      (hl),e
        page    1
        ld      (hl),d
     ; any key test
        xor     a
        in      a,(0feh)
        or      0e0h
        inc     a
        jr      nz,goboot
     ; test for cp/m
        bit     _ry7,(iy+_flag)
        ld      a,0ch
        jr      z,_1
        ld      a,0eh
_1:     call    _set_pg
        call    tst_cks
        dec     a
        jr      z,exits
     ; test for trdos (clear if not present)
        page    0eh
        call    tst_cks
        cp      3
        jr      z,_trdos
        ld      hl,0c000h
        ld      de,0c001h
        ld      bc,3ffeh
        ld      (hl),0c9h
        ldir
_trdos: page    0ch                          ; test for sinclear 48/128
        call    tst_cks
        cp      2
        jr      z,exit
        cp      5
        jr      nz,goboot
        page    8h
        call    tst_cks
        cp      4
        jr      nz,goboot
        xor     a               ; run s128
        db      0c2h            ; jp nz
exit:  ld   a,10h
exit0: ld   bc,7ffdh
       out  (c),a
       xor  a
       ld   b,0fdh
       out  (c),a
       ld   hl,0fffeh
       ld   (hl),0edh
       inc  hl
       ld   (hl),40h
       ld   bc,0ff7bh
       jp   0fffeh

exits: out	(0beh),a
       jr exit

goboot: ld      sp,SPtop
        ld      hl,pal
        call    set_pal

        call    cls##
        ld     hl,5800h
        ld     bc,406h
        ld     a,ATMcol##
        call   _f_atr##
        call   dr_atm##
        ld      bc,(mes_adr)
        bit     _nmes,(iy+_flag)
        call    nz,print##
        call    tst_tur
        jr      nc,tur_ok

        ld      bc,Mtur_off##
        call    print##
tw_l:   xor	a
	out     (0feh),a
	call    tst_tur
        jr      c,tw_l
        call    cmes##

tur_ok:       ; VG 93 exist?
;.comment @
        call    ret_rom##
        xor     a
        out     (3fh),a
        in      a,(3fh)
        inc     a
        jr      nz,menu
        set    _nodsk,(iy+_flag)
; @
menu:   call   dr_menu##
        call   w_menu##

        ld  a,e
        or  a
        jr      nz,tapel
       out  (0beh),a
       call     hclr
       ld       hl,8000h
       ld       b,1
       ld       d,l     ; de is 1
       ld       e,b
       rst      20h
       jr       nc,ga_2
derr:  or       a
       ld       hl,Mdlerr##
       jp       nz,err
       ld       hl,Mdlnsd##
       jp       err
ga_2:  ld       bc,8100h
       sbc      hl,bc
       jp       nz,8000h
       ld       a,(8000h)
       cp       0c3h
       jp       z,8000h
       ld       hl,8000h
       rst      28h
       DEFB "ATM     C",0
       jr       c,derr
       bit      _ry7,(iy+_flag)
       jp       8000h

hclr:  page 1
       ld   bc,8000
       ld   hl,0c000h
       ld   de,0c001h
       ld   (hl),0
       ldir
       ld   bc,8000
       ld   hl,0e000h
       ld   de,0e001h
       ld   (hl),0
       ldir
       ret

pexit0: xor      a
        db      0c2h            ; jp nz
pexit:  ld      a,10h
        out     (0feh),a
        push    af
        ld      hl,sinc_pal
        call    set_pal
        pop     af
        jp      exit0

tapel:  push    af
        ld      bc,Mload##
        call    print
        pop     af
     ; load tape
        cp      2
        exx
        ld      c,0feh
        jr      nz,_2
        out  (0beh),a
        push af
        call    hclr
        pop  af
        ld      c,0beh
_2:     exx
        dec     a
        jr      z,tl_s48
        dec     a
        jr      z,tl_cpmg
        ld      (iy+_ecnt),2
tl_s128:call    l_head
        sub     3
        cp      2
        jr      nc,tl_s128
        add     a,a
        add     a,a
        add     a,8h
        call    _set_pg
        call    l_body
        dec     (iy+_ecnt)
        jr      nz,tl_s128
        jp      pexit0

;----------------------------
tl_s48:

        call    l_head
        cp      2
        jr      nz,tl_s48
        page    0ch
        call    l_body
        jp      pexit

tl_cpmg:ld      (iy+_ecnt),0
tl_cpm: call    l_head
        bit     _ry7,(iy+_flag)
        ld      d,0
        jr      nz,$+3
        inc     d
        cp      d
        jr      z,tlc_cpm
        sub     5
        jr      z,tlc_cfg
        dec     a
        jr      nz,tl_cpm
        ld      a,(ecnt)
        inc     (iy+_ecnt)
        ld      a,7
        sub     d
        dec     d
        jr      nz,__2
        CP      0CH
        jr      C,__2
        INC     A
__2:    call    _set_pg
        call    l_body
        jr      tl_cpm
tlc_cpm: ld     a,d
        add     a,a
        xor     2
        add     a,0ch
        jr      __2
tlc_cfg:page    1
        ld      ix,0ff40h
        ld      de,192
        call    l_body1
        jp      exit

;ret_rom:: nop
;        ret

end
